@@ -27,17 +27,17 @@ use TokenKind::*;
 /// contents of the set is printed in the resultant error message.
 pub(crate) struct Parser<'tokens, 'src> {
   /// Source tokens
-  tokens:   &'tokens [Token<'src>],
+  tokens: &'tokens [Token<'src>],
   /// Index of the next un-parsed token
-  next:     usize,
+  next: usize,
   /// Current expected tokens
   expected: BTreeSet<TokenKind>,
 }
 
 impl<'tokens, 'src> Parser<'tokens, 'src> {
-  /// Parse `tokens` into an `Module`
-  pub(crate) fn parse(tokens: &'tokens [Token<'src>]) -> CompilationResult<'src, Module<'src>> {
-    Self::new(tokens).parse_justfile()
+  /// Parse `tokens` into an `Ast`
+  pub(crate) fn parse(tokens: &'tokens [Token<'src>]) -> CompileResult<'src, Ast<'src>> {
+    Self::new(tokens).parse_ast()
   }
 
   /// Construct a new Paser from a token stream
@@ -49,27 +49,26 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     }
   }
 
-  fn error(
-    &self,
-    kind: CompilationErrorKind<'src>,
-  ) -> CompilationResult<'src, CompilationError<'src>> {
+  fn error(&self, kind: CompileErrorKind<'src>) -> CompileResult<'src, CompileError<'src>> {
     Ok(self.next()?.error(kind))
   }
 
   /// Construct an unexpected token error with the token returned by
   /// `Parser::next`
-  fn unexpected_token(&self) -> CompilationResult<'src, CompilationError<'src>> {
-    self.error(CompilationErrorKind::UnexpectedToken {
-      expected: self.expected.iter().cloned().collect::<Vec<TokenKind>>(),
-      found:    self.next()?.kind,
+  fn unexpected_token(&self) -> CompileResult<'src, CompileError<'src>> {
+    self.error(CompileErrorKind::UnexpectedToken {
+      expected: self
+        .expected
+        .iter()
+        .copied()
+        .filter(|kind| *kind != ByteOrderMark)
+        .collect::<Vec<TokenKind>>(),
+      found: self.next()?.kind,
     })
   }
 
-  fn internal_error(
-    &self,
-    message: impl Into<String>,
-  ) -> CompilationResult<'src, CompilationError<'src>> {
-    self.error(CompilationErrorKind::Internal {
+  fn internal_error(&self, message: impl Into<String>) -> CompileResult<'src, CompileError<'src>> {
+    self.error(CompileErrorKind::Internal {
       message: message.into(),
     })
   }
@@ -78,12 +77,12 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   fn rest(&self) -> impl Iterator<Item = Token<'src>> + 'tokens {
     self.tokens[self.next..]
       .iter()
-      .cloned()
+      .copied()
       .filter(|token| token.kind != Whitespace)
   }
 
   /// The next significant token
-  fn next(&self) -> CompilationResult<'src, Token<'src>> {
+  fn next(&self) -> CompileResult<'src, Token<'src>> {
     if let Some(token) = self.rest().next() {
       Ok(token)
     } else {
@@ -107,26 +106,19 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     let mut rest = self.rest();
     for kind in kinds {
       match rest.next() {
-        Some(token) =>
+        Some(token) => {
           if token.kind != *kind {
             return false;
-          },
+          }
+        }
         None => return false,
       }
     }
     true
   }
 
-  /// Get the `n`th next significant token
-  fn get(&self, n: usize) -> CompilationResult<'src, Token<'src>> {
-    match self.rest().nth(n) {
-      Some(token) => Ok(token),
-      None => Err(self.internal_error("`Parser::get()` advanced past end of token stream")?),
-    }
-  }
-
   /// Advance past one significant token, clearing the expected token set.
-  fn advance(&mut self) -> CompilationResult<'src, Token<'src>> {
+  fn advance(&mut self) -> CompileResult<'src, Token<'src>> {
     self.expected.clear();
 
     for skipped in &self.tokens[self.next..] {
@@ -142,7 +134,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
 
   /// Return the next token if it is of kind `expected`, otherwise, return an
   /// unexpected token error
-  fn expect(&mut self, expected: TokenKind) -> CompilationResult<'src, Token<'src>> {
+  fn expect(&mut self, expected: TokenKind) -> CompileResult<'src, Token<'src>> {
     if let Some(token) = self.accept(expected)? {
       Ok(token)
     } else {
@@ -151,7 +143,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Return an unexpected token error if the next token is not an EOL
-  fn expect_eol(&mut self) -> CompilationResult<'src, ()> {
+  fn expect_eol(&mut self) -> CompileResult<'src, ()> {
     self.accept(Comment)?;
 
     if self.next_is(Eof) {
@@ -161,14 +153,14 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     self.expect(Eol).map(|_| ())
   }
 
-  fn expect_keyword(&mut self, expected: Keyword) -> CompilationResult<'src, ()> {
+  fn expect_keyword(&mut self, expected: Keyword) -> CompileResult<'src, ()> {
     let identifier = self.expect(Identifier)?;
     let found = identifier.lexeme();
 
     if expected == found {
       Ok(())
     } else {
-      Err(identifier.error(CompilationErrorKind::ExpectedKeyword {
+      Err(identifier.error(CompileErrorKind::ExpectedKeyword {
         expected: vec![expected],
         found,
       }))
@@ -177,7 +169,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
 
   /// Return an internal error if the next token is not of kind `Identifier`
   /// with lexeme `lexeme`.
-  fn presume_keyword(&mut self, keyword: Keyword) -> CompilationResult<'src, ()> {
+  fn presume_keyword(&mut self, keyword: Keyword) -> CompileResult<'src, ()> {
     let next = self.advance()?;
 
     if next.kind != Identifier {
@@ -197,7 +189,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Return an internal error if the next token is not of kind `kind`.
-  fn presume(&mut self, kind: TokenKind) -> CompilationResult<'src, Token<'src>> {
+  fn presume(&mut self, kind: TokenKind) -> CompileResult<'src, Token<'src>> {
     let next = self.advance()?;
 
     if next.kind != kind {
@@ -211,7 +203,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Return an internal error if the next token is not one of kinds `kinds`.
-  fn presume_any(&mut self, kinds: &[TokenKind]) -> CompilationResult<'src, Token<'src>> {
+  fn presume_any(&mut self, kinds: &[TokenKind]) -> CompileResult<'src, Token<'src>> {
     let next = self.advance()?;
     if !kinds.contains(&next.kind) {
       Err(self.internal_error(format!(
@@ -225,7 +217,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Accept and return a token of kind `kind`
-  fn accept(&mut self, kind: TokenKind) -> CompilationResult<'src, Option<Token<'src>>> {
+  fn accept(&mut self, kind: TokenKind) -> CompileResult<'src, Option<Token<'src>>> {
     if self.next_is(kind) {
       Ok(Some(self.advance()?))
     } else {
@@ -234,9 +226,9 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Return an error if the next token is of kind `forbidden`
-  fn forbid<F>(&self, forbidden: TokenKind, error: F) -> CompilationResult<'src, ()>
+  fn forbid<F>(&self, forbidden: TokenKind, error: F) -> CompileResult<'src, ()>
   where
-    F: FnOnce(Token) -> CompilationError,
+    F: FnOnce(Token) -> CompileError,
   {
     let next = self.next()?;
 
@@ -248,7 +240,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Accept a token of kind `Identifier` and parse into a `Name`
-  fn accept_name(&mut self) -> CompilationResult<'src, Option<Name<'src>>> {
+  fn accept_name(&mut self) -> CompileResult<'src, Option<Name<'src>>> {
     if self.next_is(Identifier) {
       Ok(Some(self.parse_name()?))
     } else {
@@ -256,7 +248,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     }
   }
 
-  fn accepted_keyword(&mut self, keyword: Keyword) -> CompilationResult<'src, bool> {
+  fn accepted_keyword(&mut self, keyword: Keyword) -> CompileResult<'src, bool> {
     let next = self.next()?;
 
     if next.kind == Identifier && next.lexeme() == keyword.lexeme() {
@@ -268,7 +260,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Accept a dependency
-  fn accept_dependency(&mut self) -> CompilationResult<'src, Option<UnresolvedDependency<'src>>> {
+  fn accept_dependency(&mut self) -> CompileResult<'src, Option<UnresolvedDependency<'src>>> {
     if let Some(recipe) = self.accept_name()? {
       Ok(Some(UnresolvedDependency {
         arguments: Vec::new(),
@@ -290,71 +282,75 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Accept and return `true` if next token is of kind `kind`
-  fn accepted(&mut self, kind: TokenKind) -> CompilationResult<'src, bool> {
+  fn accepted(&mut self, kind: TokenKind) -> CompileResult<'src, bool> {
     Ok(self.accept(kind)?.is_some())
   }
 
   /// Parse a justfile, consumes self
-  fn parse_justfile(mut self) -> CompilationResult<'src, Module<'src>> {
+  fn parse_ast(mut self) -> CompileResult<'src, Ast<'src>> {
+    fn pop_doc_comment<'src>(
+      items: &mut Vec<Item<'src>>,
+      eol_since_last_comment: bool,
+    ) -> Option<&'src str> {
+      if !eol_since_last_comment {
+        if let Some(Item::Comment(contents)) = items.last() {
+          let doc = Some(contents[1..].trim_start());
+          items.pop();
+          return doc;
+        }
+      }
+
+      None
+    }
+
     let mut items = Vec::new();
 
-    let mut doc = None;
+    let mut eol_since_last_comment = false;
+
+    self.accept(ByteOrderMark)?;
 
     loop {
       let next = self.next()?;
 
       if let Some(comment) = self.accept(Comment)? {
-        doc = Some(comment.lexeme()[1..].trim());
+        items.push(Item::Comment(comment.lexeme().trim_end()));
         self.expect_eol()?;
+        eol_since_last_comment = false;
       } else if self.accepted(Eol)? {
+        eol_since_last_comment = true;
       } else if self.accepted(Eof)? {
         break;
       } else if self.next_is(Identifier) {
         match Keyword::from_lexeme(next.lexeme()) {
-          Some(Keyword::Alias) =>
-            if self.next_are(&[Identifier, Identifier, Equals]) {
-              return Err(self.get(2)?.error(CompilationErrorKind::DeprecatedEquals));
-            } else if self.next_are(&[Identifier, Identifier, ColonEquals]) {
-              items.push(Item::Alias(self.parse_alias()?));
-            } else {
-              items.push(Item::Recipe(self.parse_recipe(doc, false)?));
-            },
-          Some(Keyword::Export) =>
-            if self.next_are(&[Identifier, Identifier, Equals]) {
-              return Err(self.get(2)?.error(CompilationErrorKind::DeprecatedEquals));
-            } else if self.next_are(&[Identifier, Identifier, ColonEquals]) {
-              self.presume_keyword(Keyword::Export)?;
-              items.push(Item::Assignment(self.parse_assignment(true)?));
-            } else {
-              items.push(Item::Recipe(self.parse_recipe(doc, false)?));
-            },
-          Some(Keyword::Set) =>
+          Some(Keyword::Alias) if self.next_are(&[Identifier, Identifier, ColonEquals]) => {
+            items.push(Item::Alias(self.parse_alias()?));
+          }
+          Some(Keyword::Export) if self.next_are(&[Identifier, Identifier, ColonEquals]) => {
+            self.presume_keyword(Keyword::Export)?;
+            items.push(Item::Assignment(self.parse_assignment(true)?));
+          }
+          Some(Keyword::Set)
             if self.next_are(&[Identifier, Identifier, ColonEquals])
               || self.next_are(&[Identifier, Identifier, Eol])
-              || self.next_are(&[Identifier, Identifier, Eof])
-            {
-              items.push(Item::Set(self.parse_set()?));
-            } else {
-              items.push(Item::Recipe(self.parse_recipe(doc, false)?));
-            },
-          _ =>
-            if self.next_are(&[Identifier, Equals]) {
-              return Err(self.get(1)?.error(CompilationErrorKind::DeprecatedEquals));
-            } else if self.next_are(&[Identifier, ColonEquals]) {
+              || self.next_are(&[Identifier, Identifier, Eof]) =>
+          {
+            items.push(Item::Set(self.parse_set()?));
+          }
+          _ => {
+            if self.next_are(&[Identifier, ColonEquals]) {
               items.push(Item::Assignment(self.parse_assignment(false)?));
             } else {
+              let doc = pop_doc_comment(&mut items, eol_since_last_comment);
               items.push(Item::Recipe(self.parse_recipe(doc, false)?));
-            },
+            }
+          }
         }
       } else if self.accepted(At)? {
+        let doc = pop_doc_comment(&mut items, eol_since_last_comment);
         items.push(Item::Recipe(self.parse_recipe(doc, true)?));
       } else {
         return Err(self.unexpected_token()?);
       }
-
-      if next.kind != Comment {
-        doc = None;
-      }
     }
 
     if self.next != self.tokens.len() {
@@ -363,7 +359,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
         self.tokens.len() - self.next,
       ))?)
     } else {
-      Ok(Module {
+      Ok(Ast {
         warnings: Vec::new(),
         items,
       })
@@ -371,7 +367,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse an alias, e.g `alias name := target`
-  fn parse_alias(&mut self) -> CompilationResult<'src, Alias<'src, Name<'src>>> {
+  fn parse_alias(&mut self) -> CompileResult<'src, Alias<'src, Name<'src>>> {
     self.presume_keyword(Keyword::Alias)?;
     let name = self.parse_name()?;
     self.presume_any(&[Equals, ColonEquals])?;
@@ -381,7 +377,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse an assignment, e.g. `foo := bar`
-  fn parse_assignment(&mut self, export: bool) -> CompilationResult<'src, Assignment<'src>> {
+  fn parse_assignment(&mut self, export: bool) -> CompileResult<'src, Assignment<'src>> {
     let name = self.parse_name()?;
     self.presume_any(&[Equals, ColonEquals])?;
     let value = self.parse_expression()?;
@@ -394,54 +390,65 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse an expression, e.g. `1 + 2`
-  fn parse_expression(&mut self) -> CompilationResult<'src, Expression<'src>> {
+  fn parse_expression(&mut self) -> CompileResult<'src, Expression<'src>> {
     if self.accepted_keyword(Keyword::If)? {
-      let lhs = self.parse_expression()?;
-
-      let inverted = self.accepted(BangEquals)?;
-
-      if !inverted {
-        self.expect(EqualsEquals)?;
-      }
-
-      let rhs = self.parse_expression()?;
-
-      self.expect(BraceL)?;
-
-      let then = self.parse_expression()?;
-
-      self.expect(BraceR)?;
-
-      self.expect_keyword(Keyword::Else)?;
-
-      self.expect(BraceL)?;
-
-      let otherwise = self.parse_expression()?;
-
-      self.expect(BraceR)?;
-
-      return Ok(Expression::Conditional {
-        lhs: Box::new(lhs),
-        rhs: Box::new(rhs),
-        then: Box::new(then),
-        otherwise: Box::new(otherwise),
-        inverted,
-      });
-    }
-
-    let value = self.parse_value()?;
-
-    if self.accepted(Plus)? {
-      let lhs = Box::new(value);
-      let rhs = Box::new(self.parse_expression()?);
-      Ok(Expression::Concatination { lhs, rhs })
+      self.parse_conditional()
     } else {
-      Ok(value)
+      let value = self.parse_value()?;
+
+      if self.accepted(Plus)? {
+        let lhs = Box::new(value);
+        let rhs = Box::new(self.parse_expression()?);
+        Ok(Expression::Concatination { lhs, rhs })
+      } else {
+        Ok(value)
+      }
     }
   }
 
+  /// Parse a conditional, e.g. `if a == b { "foo" } else { "bar" }`
+  fn parse_conditional(&mut self) -> CompileResult<'src, Expression<'src>> {
+    let lhs = self.parse_expression()?;
+
+    let operator = if self.accepted(BangEquals)? {
+      ConditionalOperator::Inequality
+    } else if self.accepted(EqualsTilde)? {
+      ConditionalOperator::RegexMatch
+    } else {
+      self.expect(EqualsEquals)?;
+      ConditionalOperator::Equality
+    };
+
+    let rhs = self.parse_expression()?;
+
+    self.expect(BraceL)?;
+
+    let then = self.parse_expression()?;
+
+    self.expect(BraceR)?;
+
+    self.expect_keyword(Keyword::Else)?;
+
+    let otherwise = if self.accepted_keyword(Keyword::If)? {
+      self.parse_conditional()?
+    } else {
+      self.expect(BraceL)?;
+      let otherwise = self.parse_expression()?;
+      self.expect(BraceR)?;
+      otherwise
+    };
+
+    Ok(Expression::Conditional {
+      lhs: Box::new(lhs),
+      rhs: Box::new(rhs),
+      then: Box::new(then),
+      otherwise: Box::new(otherwise),
+      operator,
+    })
+  }
+
   /// Parse a value, e.g. `(bar)`
-  fn parse_value(&mut self) -> CompilationResult<'src, Expression<'src>> {
+  fn parse_value(&mut self) -> CompileResult<'src, Expression<'src>> {
     if self.next_is(StringToken) {
       Ok(Expression::StringLiteral {
         string_literal: self.parse_string_literal()?,
@@ -459,7 +466,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       };
 
       if contents.starts_with("#!") {
-        return Err(next.error(CompilationErrorKind::BacktickShebang));
+        return Err(next.error(CompileErrorKind::BacktickShebang));
       }
 
       Ok(Expression::Backtick { contents, token })
@@ -485,7 +492,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse a string literal, e.g. `"FOO"`
-  fn parse_string_literal(&mut self) -> CompilationResult<'src, StringLiteral<'src>> {
+  fn parse_string_literal(&mut self) -> CompileResult<'src, StringLiteral<'src>> {
     let token = self.expect(StringToken)?;
 
     let kind = StringKind::from_string_or_backtick(token)?;
@@ -510,13 +517,13 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
             'r' => cooked.push('\r'),
             't' => cooked.push('\t'),
             '\\' => cooked.push('\\'),
-            '\n' => {},
+            '\n' => {}
             '"' => cooked.push('"'),
             other => {
               return Err(
-                token.error(CompilationErrorKind::InvalidEscapeSequence { character: other }),
+                token.error(CompileErrorKind::InvalidEscapeSequence { character: other }),
               );
-            },
+            }
           }
           escape = false;
         } else if c == '\\' {
@@ -534,12 +541,12 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse a name from an identifier token
-  fn parse_name(&mut self) -> CompilationResult<'src, Name<'src>> {
+  fn parse_name(&mut self) -> CompileResult<'src, Name<'src>> {
     self.expect(Identifier).map(Name::from_identifier)
   }
 
   /// Parse sequence of comma-separated expressions
-  fn parse_sequence(&mut self) -> CompilationResult<'src, Vec<Expression<'src>>> {
+  fn parse_sequence(&mut self) -> CompileResult<'src, Vec<Expression<'src>>> {
     self.presume(ParenL)?;
 
     let mut elements = Vec::new();
@@ -562,7 +569,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     &mut self,
     doc: Option<&'src str>,
     quiet: bool,
-  ) -> CompilationResult<'src, UnresolvedRecipe<'src>> {
+  ) -> CompileResult<'src, UnresolvedRecipe<'src>> {
     let name = self.parse_name()?;
 
     let mut positional = Vec::new();
@@ -583,7 +590,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       let variadic = self.parse_parameter(kind)?;
 
       self.forbid(Identifier, |token| {
-        token.error(CompilationErrorKind::ParameterFollowsVariadicParameter {
+        token.error(CompileErrorKind::ParameterFollowsVariadicParameter {
           parameter: token.lexeme(),
         })
       })?;
@@ -601,24 +608,41 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       dependencies.push(dependency);
     }
 
+    let priors = dependencies.len();
+
+    if self.accepted(AmpersandAmpersand)? {
+      let mut subsequents = Vec::new();
+
+      while let Some(subsequent) = self.accept_dependency()? {
+        subsequents.push(subsequent);
+      }
+
+      if subsequents.is_empty() {
+        return Err(self.unexpected_token()?);
+      }
+
+      dependencies.append(&mut subsequents);
+    }
+
     self.expect_eol()?;
 
     let body = self.parse_body()?;
 
     Ok(Recipe {
-      private: name.lexeme().starts_with('_'),
-      shebang: body.first().map(Line::is_shebang).unwrap_or(false),
       parameters: positional.into_iter().chain(variadic).collect(),
+      private: name.lexeme().starts_with('_'),
+      shebang: body.first().map_or(false, Line::is_shebang),
+      priors,
+      body,
+      dependencies,
       doc,
       name,
       quiet,
-      dependencies,
-      body,
     })
   }
 
   /// Parse a recipe parameter
-  fn parse_parameter(&mut self, kind: ParameterKind) -> CompilationResult<'src, Parameter<'src>> {
+  fn parse_parameter(&mut self, kind: ParameterKind) -> CompileResult<'src, Parameter<'src>> {
     let export = self.accepted(Dollar)?;
 
     let name = self.parse_name()?;
@@ -630,15 +654,15 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     };
 
     Ok(Parameter {
-      name,
-      kind,
       default,
       export,
+      kind,
+      name,
     })
   }
 
   /// Parse the body of a recipe
-  fn parse_body(&mut self) -> CompilationResult<'src, Vec<Line<'src>>> {
+  fn parse_body(&mut self) -> CompileResult<'src, Vec<Line<'src>>> {
     let mut lines = Vec::new();
 
     if self.accepted(Indent)? {
@@ -670,7 +694,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       }
     }
 
-    while lines.last().map(Line::is_empty).unwrap_or(false) {
+    while lines.last().map_or(false, Line::is_empty) {
       lines.pop();
     }
 
@@ -678,7 +702,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse a boolean setting value
-  fn parse_set_bool(&mut self) -> CompilationResult<'src, bool> {
+  fn parse_set_bool(&mut self) -> CompileResult<'src, bool> {
     if !self.accepted(ColonEquals)? {
       return Ok(true);
     }
@@ -690,9 +714,9 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     } else if Keyword::False == identifier.lexeme() {
       false
     } else {
-      return Err(identifier.error(CompilationErrorKind::ExpectedKeyword {
+      return Err(identifier.error(CompileErrorKind::ExpectedKeyword {
         expected: vec![Keyword::True, Keyword::False],
-        found:    identifier.lexeme(),
+        found: identifier.lexeme(),
       }));
     };
 
@@ -700,12 +724,18 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse a setting
-  fn parse_set(&mut self) -> CompilationResult<'src, Set<'src>> {
+  fn parse_set(&mut self) -> CompileResult<'src, Set<'src>> {
     self.presume_keyword(Keyword::Set)?;
     let name = Name::from_identifier(self.presume(Identifier)?);
     let lexeme = name.lexeme();
 
-    if Keyword::DotenvLoad == lexeme {
+    if Keyword::AllowDuplicateRecipes == lexeme {
+      let value = self.parse_set_bool()?;
+      return Ok(Set {
+        value: Setting::AllowDuplicateRecipes(value),
+        name,
+      });
+    } else if Keyword::DotenvLoad == lexeme {
       let value = self.parse_set_bool()?;
       return Ok(Set {
         value: Setting::DotenvLoad(value),
@@ -723,6 +753,12 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
         value: Setting::PositionalArguments(value),
         name,
       });
+    } else if Keyword::WindowsPowershell == lexeme {
+      let value = self.parse_set_bool()?;
+      return Ok(Set {
+        value: Setting::WindowsPowerShell(value),
+        name,
+      });
     }
 
     self.expect(ColonEquals)?;
@@ -747,11 +783,11 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       self.expect(BracketR)?;
 
       Ok(Set {
-        value: Setting::Shell(setting::Shell { command, arguments }),
+        value: Setting::Shell(setting::Shell { arguments, command }),
         name,
       })
     } else {
-      Err(name.error(CompilationErrorKind::UnknownSetting {
+      Err(name.error(CompileErrorKind::UnknownSetting {
         setting: name.lexeme(),
       }))
     }
@@ -763,7 +799,7 @@ mod tests {
   use super::*;
 
   use pretty_assertions::assert_eq;
-  use CompilationErrorKind::*;
+  use CompileErrorKind::*;
 
   macro_rules! test {
     {
@@ -817,14 +853,14 @@ mod tests {
     line: usize,
     column: usize,
     length: usize,
-    kind: CompilationErrorKind,
+    kind: CompileErrorKind,
   ) {
     let tokens = Lexer::lex(src).expect("Lexing failed in parse test...");
 
     match Parser::parse(&tokens) {
       Ok(_) => panic!("Parsing unexpectedly succeeded"),
       Err(have) => {
-        let want = CompilationError {
+        let want = CompileError {
           token: Token {
             kind: have.token.kind,
             src,
@@ -836,7 +872,7 @@ mod tests {
           kind,
         };
         assert_eq!(have, want);
-      },
+      }
     }
   }
 
@@ -1076,6 +1112,12 @@ mod tests {
     tree: (justfile (recipe foo (deps (bar (+ 'a' 'b') (+ 'c' 'd'))))),
   }
 
+  test! {
+    name: recipe_subsequent,
+    text: "foo: && bar",
+    tree: (justfile (recipe foo (sups bar))),
+  }
+
   test! {
     name: recipe_line_single,
     text: "foo:\n bar",
@@ -1097,11 +1139,17 @@ mod tests {
   test! {
     name: comment,
     text: "# foo",
-    tree: (justfile),
+    tree: (justfile (comment "# foo")),
   }
 
   test! {
-    name: comment_alias,
+    name: comment_before_alias,
+    text: "# foo\nalias x := y",
+    tree: (justfile (comment "# foo") (alias x y)),
+  }
+
+  test! {
+    name: comment_after_alias,
     text: "alias x := y # foo",
     tree: (justfile (alias x y)),
   }
@@ -1166,7 +1214,7 @@ mod tests {
       x := y
       bar:
     ",
-    tree: (justfile (assignment x y) (recipe bar)),
+    tree: (justfile (comment "# foo") (assignment x y) (recipe bar)),
   }
 
   test! {
@@ -1176,7 +1224,7 @@ mod tests {
 
       bar:
     ",
-    tree: (justfile (recipe bar)),
+    tree: (justfile (comment "# foo") (recipe bar)),
   }
 
   test! {
@@ -1672,6 +1720,12 @@ mod tests {
     tree: (justfile (set dotenv_load true)),
   }
 
+  test! {
+    name: set_allow_duplicate_recipes_implicit,
+    text: "set allow-duplicate-recipes",
+    tree: (justfile (set allow_duplicate_recipes true)),
+  }
+
   test! {
     name: set_dotenv_load_true,
     text: "set dotenv-load := true",
@@ -1738,6 +1792,24 @@ mod tests {
     tree: (justfile (set shell "bash" "-cu" "-l")),
   }
 
+  test! {
+    name: set_windows_powershell_implicit,
+    text: "set windows-powershell",
+    tree: (justfile (set windows_powershell true)),
+  }
+
+  test! {
+    name: set_windows_powershell_true,
+    text: "set windows-powershell := true",
+    tree: (justfile (set windows_powershell true)),
+  }
+
+  test! {
+    name: set_windows_powershell_false,
+    text: "set windows-powershell := false",
+    tree: (justfile (set windows_powershell false)),
+  }
+
   test! {
     name: conditional,
     text: "a := if b == c { d } else { e }",
@@ -1853,10 +1925,13 @@ mod tests {
     name:   missing_eol,
     input:  "a b c: z =",
     offset:  9,
-    line:   0,
-    column: 9,
-    width:  1,
-    kind:   UnexpectedToken{expected: vec![Comment, Eof, Eol, Identifier, ParenL], found: Equals},
+    line:    0,
+    column:  9,
+    width:   1,
+    kind:    UnexpectedToken{
+      expected: vec![AmpersandAmpersand, Comment, Eof, Eol, Identifier, ParenL],
+      found: Equals
+    },
   }
 
   error! {
@@ -2107,7 +2182,7 @@ mod tests {
     kind: FunctionArgumentCountMismatch {
       function: "arch",
       found: 1,
-      expected: 0,
+      expected: 0..0,
     },
   }
 
@@ -2121,7 +2196,7 @@ mod tests {
     kind: FunctionArgumentCountMismatch {
       function: "env_var",
       found: 0,
-      expected: 1,
+      expected: 1..1,
     },
   }
 
@@ -2135,7 +2210,35 @@ mod tests {
     kind: FunctionArgumentCountMismatch {
       function: "env_var_or_default",
       found: 1,
-      expected: 2,
+      expected: 2..2,
+    },
+  }
+
+  error! {
+    name: function_argument_count_binary_plus,
+    input: "x := join('foo')",
+    offset: 5,
+    line: 0,
+    column: 5,
+    width: 4,
+    kind: FunctionArgumentCountMismatch {
+      function: "join",
+      found: 1,
+      expected: 2..usize::MAX,
+    },
+  }
+
+  error! {
+    name: function_argument_count_ternary,
+    input: "x := replace('foo')",
+    offset: 5,
+    line: 0,
+    column: 5,
+    width: 7,
+    kind: FunctionArgumentCountMismatch {
+      function: "replace",
+      found: 1,
+      expected: 3..3,
     },
   }
 }
